{*****************************************************
  PFSS Detail Display - GUI    ver 1.3.96   Jan, 2005
  Office of Information Field Office
  Department of Veterans Affairs
  Hines,Illinois

  Developers: Helen Corkwell, Ken Nelson
  Description: GUI to view and print Prosthetics Billing Information
  Requires: RMPR*3.0*71 - RMPR*3.0*59 - RMPR*3.0*96 - RMPR*3.0*149

*******************************************************}

unit PROS_PFFSDetailDisplay_Main;

{$DEFINE BILLING} //Use for Billing version

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls,MFunStr, MyGrids, SortGrid, SRGrad, ESBDates, Buttons, Trpcb,
  ExtCtrls, ShellAPI, Rpcconf1, Menus, NumEdit, Registry, FileCtrl, ActnList,
  Printers, ProgBar, StrUtils;

type
  TfrmViewBilling = class(TForm)
    imgGradient: TSRGradient;
    grdMain: TSortGrid;
    edtBegDate: TEdit;
    edtEndDate: TEdit;
    dtpkrBegDate: TDateTimePicker;
    dtpkrEndDate: TDateTimePicker;
    btnDisplay: TBitBtn;
    bnrMain: TLabel;
    lblBegDate: TLabel;
    lblEndDate: TLabel;
    btnPrintGrid: TBitBtn;
    dlgPrint: TPrintDialog;
    btnExcel: TBitBtn;
    btnClose: TBitBtn;
    lblRecFound: TLabel;
    lblDisRecFound: TLabel;
    mnuViewBilling: TMainMenu;
    mnuItemHelp: TMenuItem;
    mnuItemFile: TMenuItem;
    mnuItemBegDate: TMenuItem;
    mnuItemEndDate: TMenuItem;
    mnuItemDisplay: TMenuItem;
    menuItemPrint: TMenuItem;
    mnuItemExcel: TMenuItem;
    mnuItemExit: TMenuItem;
    mnuSep1: TMenuItem;
    mnuSep2: TMenuItem;
    stsbrMain: TStatusBar;
    mnuItemContents: TMenuItem;
    mnuSep3: TMenuItem;
    mnuSep4: TMenuItem;
    mnuItemSec508: TMenuItem;
    btnClear: TBitBtn;
    mnuItemAbout: TMenuItem;
    pnlProgressBar: TPanel;
    grdMiniInsurance: TSortGrid;
    grdMiniDis: TSortGrid;
    btnPtDis: TBitBtn;
    Label1: TLabel;
    btnMiniInsurance: TBitBtn;
    Label2: TLabel;
    lblExcelPath: TLabel;
    procedure edtBegDateChange(Sender: TObject);
    procedure dtpkrBegDateChange(Sender: TObject);
    procedure edtEndDateChange(Sender: TObject);
    procedure dtpkrEndDateChange(Sender: TObject);
    procedure btnDisplayClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btnPrintGridClick(Sender: TObject);
    procedure dtpkrBegDateEnter(Sender: TObject);
    procedure btnExcelClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CheckSec508;
    procedure mnuItemBegDateClick(Sender: TObject);
    procedure mnuItemDisplayClick(Sender: TObject);
    procedure menuItemPrintClick(Sender: TObject);
    procedure mnuItemExcelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DisplayHint(Sender: TObject);
    procedure mnuItemAboutClick(Sender: TObject);
    procedure mnuItemContentsClick(Sender: TObject);
    procedure mnuItemExitClick(Sender: TObject);
    procedure mnuItemEndDateClick(Sender: TObject);
    procedure dtpkrBegDateKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure dtpkrEndDateKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure dtpkrEndDateEnter(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnuItemSec508Click(Sender: TObject);
    procedure grdMainGetCellFormat(Sender: TObject; Col, Row: Integer;
      State: TGridDrawState; var FormatOptions: TFormatOptions;
      var CheckBox, Combobox, Ellipsis: Boolean);
    procedure grdMainGetSortStyle(Sender: TObject; Col: Integer;
      var SortStyle: TSortStyle);
    procedure btnClearClick(Sender: TObject);
    procedure grdMainDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure btnPtDisClick(Sender: TObject);
    procedure grdMainClick(Sender: TObject);
    procedure btnMiniInsuranceClick(Sender: TObject);
    procedure grdMiniInsuranceGetSortStyle(Sender: TObject; Col: Integer;
      var SortStyle: TSortStyle);
    procedure grdMiniInsuranceDrawCell(Sender: TObject; ACol,
      ARow: Integer; Rect: TRect; State: TGridDrawState);
  private
    ExcelFile : string;         //added with RMPR*149
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmViewBilling: TfrmViewBilling;
  Sec508,                  //Sec508 status from registry
  MDate1, MDate2, IEN660 : string; // First and second 'Mumps' dates for search
  BegDtpkrDate, EndDtpkrDate : TDateTime;

implementation


{$IFDEF BILLING}  //

uses Pros_Billing_Menu, PROS_Billing_Menu_About;

{$ELSE}

uses Pros_Menu, PROS_Menu_About;

{$ENDIF}

//uses PROS_NPPDDetailDisplay_About;

{$R *.DFM}

procedure TfrmViewBilling.CheckSec508; //Checks the SEC508 registry Key
var
  reg : TRegistry;
begin
   reg := TRegistry.Create;
   reg.rootkey := HKEY_LOCAL_MACHINE;
   reg.OpenKey('\SOFTWARE\VISTA\PROS\ViewBilling',False);
   Sec508 := reg.ReadString('Sec508');  //Assign result to global
   reg.Free;
end;


procedure TfrmViewBilling.DisplayHint(Sender: TObject);
begin
  stsbrMain.SimpleText := Application.Hint; //For Sec508
end;


procedure DeleteFiles(sMask, sPath: string);
var
  SearchRec: TSearchRec;
  Found: Integer;
begin
  sPath := (sPath);
  Found := SysUtils.FindFirst(sPath + sMask, faAnyFile, SearchRec); 
  try
    while (Found = 0) do
    begin
      if not (SearchRec.Attr and faDirectory > 0) then
        SysUtils.DeleteFile(sPath + SearchRec.Name);
      Found := SysUtils.FindNext(SearchRec);
    end;
  finally
    SysUtils.FindClose(SearchRec);
  end;
end; // Usage:  DeleteFiles('*.txt', 'c:\myfiles');



//First Date
procedure TfrmViewBilling.edtBegDateChange(Sender: TObject);
var
  s1,d1,d2,d3 : string;        //Creates the Jun 01,2002 display date
begin                          //from the tossed calendar pick
s1 := '';                      //Clears everything
  if edtBegDate.Text <> '' then
   begin
        d1 := Date2ShortMonth(dtpkrBegDate.Date);
       if Date2Day(dtpkrBegDate.Date) < 10 then  //Pad with 0 if needed
        d2:= '0'+ IntToStr(Date2Day(dtpkrBegDate.Date))
       else
        d2 := IntToStr(Date2Day(dtpkrBegDate.Date));  //Or not
        d3 := IntToStr(Date2Year(dtpkrBegDate.Date));

   s1 := d1+' '+d2+', '+d3; //Assembles the pieces
   edtBegDate.text := s1         //Display

   end;
end;

procedure TfrmViewBilling.dtpkrBegDateChange(Sender: TObject);
var
d1,d2,d3,d4 : string;         //Gets the user input from the calendar
begin
      MDate1 := '';
      if (Date2Year(dtpkrBegDate.Date) < 1900) then  //Checks century
    d1 := '1';
      if (Date2Year(dtpkrBegDate.Date) >= 1900) and
         (Date2Year(dtpkrBegDate.Date) < 2000) then
    d1 := '2';
      if (Date2Year(dtpkrBegDate.Date) >= 2000) then
    d1 := '3';

    d2 := Copy(IntToStr(Date2Year(dtpkrBegDate.Date)),3,2); //Gets year

      if Date2Month(dtpkrBegDate.Date) < 10 then //Pad month with 0 if needed
    d3:= '0'+ IntToStr(Date2Month(dtpkrBegDate.Date))
       else
    d3 := IntToStr(Date2Month(dtpkrBegDate.Date));

     if Date2Day(dtpkrBegDate.Date) < 10 then  //Pad day with 0 if needed
    d4:= '0'+ IntToStr(Date2Day(dtpkrBegDate.Date))
       else
    d4 := IntToStr(Date2Day(dtpkrBegDate.Date));

MDate1 := d1 + d2+ d3 + d4;                    //Assemble MUMPS date style
edtBegDate.Text := Date2Str(dtpkrBegDate.Date);  //Toss the date to Edit

end;


//*************************

//Second Date
procedure TfrmViewBilling.edtEndDateChange(Sender: TObject);
var
  s1,d1,d2,d3: string;        //Creates the Jun 01,2002 display date
begin                         //from the tossed calendar pick
s1 := '';                     //Clears everything
  if edtEndDate.Text <> '' then
   begin
        d1 := Date2ShortMonth(dtpkrEndDate.Date);
       if Date2Day(dtpkrEndDate.Date) < 10 then  //Pad with 0 if needed
        d2:= '0'+ IntToStr(Date2Day(dtpkrEndDate.Date))
       else
        d2 := IntToStr(Date2Day(dtpkrEndDate.Date));  //Or not
        d3 := IntToStr(Date2Year(dtpkrEndDate.Date));

   s1 := d1+' '+d2+', '+d3;    //Assembles the pieces
   edtEndDate.text := s1          //Display

   end;
end;


procedure TfrmViewBilling.dtpkrEndDateChange(Sender: TObject);
var
d1,d2,d3,d4 : string;
                            //Gets the user input from the calendar
begin
      MDate2 := '';
      if (Date2Year(dtpkrEndDate.Date) < 1900) then  //Checks century
    d1 := '1';
      if (Date2Year(dtpkrEndDate.Date) >= 1900) and
         (Date2Year(dtpkrEndDate.Date) < 2000) then
    d1 := '2';
      if (Date2Year(dtpkrEndDate.Date) >= 2000) then
    d1 := '3';

    d2 := Copy(IntToStr(Date2Year(dtpkrEndDate.Date)),3,2);//Gets year

        if Date2Month(dtpkrEndDate.Date) < 10 then //Pad month with 0 if needed
    d3:= '0'+ IntToStr(Date2Month(dtpkrEndDate.Date))
       else
    d3 := IntToStr(Date2Month(dtpkrEndDate.Date));

        if Date2Day(dtpkrEndDate.Date) < 10 then  //Pad day with 0 if needed
    d4:= '0'+ IntToStr(Date2Day(dtpkrEndDate.Date))
       else
    d4 := IntToStr(Date2Day(dtpkrEndDate.Date));

  MDate2 := d1 + d2+ d3 + d4;          //Assemble MUMPS date style
  edtEndDate.Text := Date2Str(dtpkrBegDate.Date); //Toss the date to Edit
 end;


procedure TfrmViewBilling.btnDisplayClick(Sender: TObject);  //Fill the grid
var
i : integer;

  ani: TAnimationThread;
  r: TRect;

begin
  lblDisRecFound.Caption := '0';
  if frmMenu.brkrRPC.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else
    begin
     if (MDate1 = '') or (MDate2 = '') then
       MessageDlg('Please select a date range.', mtInformation,
      [mbOk], 0)
     else
   begin
    if StrToInt(MDate1) <= StrToInt(Mdate2) then //Test if Date1 before Date2
   begin
     begin
      for i :=1 to grdMain.RowCount -1 do   //If selected will clear the grid
      grdMain.Rows[i].Clear;
      grdMain.Row := 1;
      grdMain.Cells[0,1] :='';
     end;
     begin
      for i :=1 to grdMiniInsurance.RowCount -1 do   //If selected will clear the grid
      grdMiniInsurance.Rows[i].Clear;
      grdMiniInsurance.Row := 1;
      grdMiniInsurance.Cells[0,1] :='';
      grdMiniInsurance.RowCount := 2;
     end;
     begin
      for i :=1 to grdMiniDis.RowCount -1 do   //If selected will clear the grid
      grdMiniDis.Rows[i].Clear;
      grdMiniDis.Row := 1;
      grdMiniDis.Cells[0,1] :='';
      grdMiniDis.RowCount := 2;
     end;

  if DaysApart (dtpkrBegDate.Date,dtpkrEndDate.Date) > 100 //Date Range checking
  then  MessageDlg('You have selected a range'+#13+'of more than 100 days.', mtInformation,
      [mbOk], 0)
   else

begin
    btnDisplay.Caption := 'Searching';
    frmMenu.brkrRPC.CreateContext('RMPR PFFS GUI');
    frmMenu.brkrRPC.RemoteProcedure := 'RMPR PFFS LIST';
    frmMenu.brkrRPC.Param[0].Value := MDate1;
    frmMenu.brkrRPC.Param[0].PType := literal;
    frmMenu.brkrRPC.Param[1].Value := MDate2;
    frmMenu.brkrRPC.Param[1].PType := literal;
    frmMenu.brkrRPC.RPCTimeLimit := 600;    //10 Min ?!?

    Try
    //
     pnlProgressBar.Visible := true;
     r := pnlProgressBar.clientrect;
     InflateRect( r, - pnlProgressBar.bevelwidth, - pnlProgressBar.bevelwidth );
     ani := TanimationThread.Create( pnlProgressBar, r, pnlProgressBar.color, clBlue, 25 );
     Application.ProcessMessages;
     frmMenu.brkrRPC.Call;
     //Sleep( 1000 );  {replace with query.Open or such}
     pnlProgressBar.Visible := False;
     ani.Terminate;
    except
     on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
     end;

    lblDisRecFound.Caption := IntToStr(frmMenu.brkrRPC.Results.Count);
    btnDisplay.Caption := '&Display';

 //Sets the grid rows to match result for sort to work correct
 //The Column moving is OFF by default because of a error if empty
 //After data is retrieved, sets to ON.
  if frmMenu.brkrRPC.Results.Count > 1 then
   begin
    grdMain.RowCount := frmMenu.brkrRPC.Results.Count +1;
    grdMain.Options := grdMain.Options+[goColMoving]; //ON
   end;

 for i:= 0 to frmMenu.brkrRPC.Results.Count -1  do
   begin

   with grdMain do           // Sets column headers on new DISPLAY
 begin
      Cells[0,0]  := 'Site';
      Cells[1,0]  := 'Create Date';
      Cells[2,0]  := 'Delivery Date';
      Cells[3,0]  := 'Patient';
      Cells[4,0]  := 'SSN';
      Cells[5,0]  := 'Insurance';
      Cells[6,0]  := 'Coding Errors';
      Cells[7,0]  := 'Item';
      Cells[8,0]  := 'Qty';
      Cells[9,0]  := 'Total Cost';
      Cells[10,0] := 'HCPCS';
      Cells[11,0] := 'HCPCS Desc';
      Cells[12,0] := 'ICD9';
      Cells[13,0] := 'ICD9 Desc';
      Cells[14,0] := 'IEN';
      Cells[15,0] := 'Coding Flag';

                                                             //Field/RPC position
   Cells[0,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',11);//Site  11
   Cells[1,i+1]  :=
      Date2Str(EnhStr2Date (Piece(frmMenu.brkrRPC.Results[i],'^',1)));//Create Date 1
     if Cells[1,i+1] = '' then Cells[1,i+1] := '01/01/0001'; //Adds date if none
   Cells[2,i+1]  :=
      Date2Str(EnhStr2Date (Piece(frmMenu.brkrRPC.Results[i],'^',20)));//Delivery Date 20
     if Cells[2,i+1] = '' then Cells[2,i+1] := '01/01/0001'; //Adds date if none
   Cells[3,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',2);  //Pat 2
   Cells[4,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',15); //SSN 15
   Cells[5,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',5);  //Insurance 5
   Cells[6,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',9);  //Coding Errors 9
   Cells[7,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',8);  //Item 8
   Cells[8,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',4);  //Qty 4
   Cells[9,i+1]  := Piece(frmMenu.brkrRPC.Results[i],'^',7);  //Total Cost 7
   Cells[10,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',3);  //HCPCS 3
   Cells[11,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',17); //HCPCS Desc 17
   Cells[12,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',18); //ICD9 18
   Cells[13,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',12); //ICD9 Desc 12
   Cells[14,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',16); //IEN
   Cells[15,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',21); //Error Code
   end
  end;


  end;
 end
  else  //Result if date1 NOT before date2
      MessageDlg('The Starting Date must be before the Ending Date', mtInformation,
      [mbOk], 0);
  end;
 end;
end;


procedure TfrmViewBilling.FormActivate(Sender: TObject);
begin
Application.HelpFile := 'PROS_ViewBillingInformation.HLP'; //Needed here
Application.OnHint := DisplayHint;  //Displays hints in Status Bar
frmViewBilling.Caption := 'View Prosthetic Billing Information . . .    You are logged onto '+frmMenu.brkrRPC.Server;

with grdMain do               // Sets column headers at start of App
 begin
      Cells[0,0]  := 'Site';
      Cells[1,0]  := 'Create Date';
      Cells[2,0]  := 'Delivery Date';
      Cells[3,0]  := 'Patient';
      Cells[4,0]  := 'SSN';
      Cells[5,0]  := 'Insurance';
      Cells[6,0]  := 'Coding Errors';
      Cells[7,0]  := 'Item';
      Cells[8,0]  := 'Qty';
      Cells[9,0]  := 'Total Cost';
      Cells[10,0] := 'HCPCS';
      Cells[11,0] := 'HCPCS Desc';
      Cells[12,0] := 'ICD9';
      Cells[13,0] := 'ICD9 Desc';
      Cells[14,0] := 'IEN';
      Cells[15,0] := 'Coding Flag';


 end;

 with grdMiniDis do       // Mini Grid header
   begin
      Cells[0,0] := 'Disability';
      Cells[1,0] := ' %';
      Cells[2,0] := 'SC';
   end;
 with grdMiniInsurance do       // Mini Grid header
   begin
      Cells[0,0] := 'Insurance';
      Cells[1,0] := 'COB';
      Cells[2,0] := 'Sub ID';
      Cells[3,0] := 'Group';
      Cells[4,0] := 'Holder';
      Cells[5,0] := 'Effective';
      Cells[6,0] := 'Expires';
   end;

end;

procedure TfrmViewBilling.btnPrintGridClick(Sender: TObject);  //Print form
var
poOriginal : TPrinterOrientation;
i : integer;
begin
  if frmMenu.brkrRPC.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else
  begin
  if grdMain.Cells[1,1] = '' then
      MessageDlg('Please DISPLAY the data to print.', mtInformation,
      [mbOk], 0)
  else

  begin

     for i := 1 to grdMain.RowCount do  // revert blank dates back to blank
   begin
    if grdMain.Cells[1,i] = '01/01/0001' then grdMain.Cells[1,i] := '';
    if grdMain.Cells[2,i] = '01/01/0001' then grdMain.Cells[2,i] := '';
  end;

  poOriginal := Printer.Orientation;   //Save original Printer Orientation
  Printer.Orientation := poLandscape;  //Set to landscape
   if dlgPrint.Execute = true then
    begin
     grdMain.PrintOptions.PageTitle :=      //Form HEADER
      'PROSTHETIC BILLING INFORMATION    Source: '+frmMenu.brkrRPC.Server+'      '+edtBegDate.Text+'  to  '+
       edtEndDate.Text+'           Printed: '+Date2Str(Now);
     grdMain.PrintOptions.ToRow := grdMain.RowCount;
     grdMain.Print;     //PRINT

  for i := 1 to grdMain.RowCount do  // make blank dates sortable
   begin
    if grdMain.Cells[1,i] = '' then grdMain.Cells[1,i] := '01/01/0001';
    if grdMain.Cells[2,i] = '' then grdMain.Cells[2,i] := '01/01/0001';
  end;

  end; //Dialog
  Printer.Orientation := poOriginal;  //reset to original
 end;
 end;
end;

procedure TfrmViewBilling.btnExcelClick(Sender: TObject); //Save and launch as Excel
var
    i : integer;
    DIR : String;
    buttonSelected : Integer;
 begin
  if frmMenu.brkrRPC.Connected = false then      //Checks if user is logged on
      MessageDlg('Please Sign On first.', mtInformation,
      [mbOk], 0)
  else

  begin

   if fileexists(excelfile) then     //modified with RMPR*149 to add securtiy enhancements
    begin
      if SysUtils.DeleteFile(ExcelFile) = False then
        raise Exception.Create('Please close the temporary Excel (.csv) file before continuing')     //rmpr*136
      else
        lblExcelPath.Caption := '';
    end;

  begin
  if grdMain.Cells[1,1] = '' then
      MessageDlg('Please DISPLAY the data to send to Excel.', mtInformation,
      [mbOk], 0)
  else
  begin
     begin    //Check for default CSV folder and create it
      if not DirectoryExists('c:\ViewBillingDownload') then
      if not CreateDir('c:\ViewBillingDownload') then
      raise Exception.Create('Cannot create c:\ViewBillingDownload');
     end;

     DIR := 'c:\ViewBillingDownload';

      buttonSelected := Application.MessageBox('The data you are about to export may'+
      ' contain Patient Identifiable Information, which requires appropriate measures '+
      'to ensure its confidentiality.  Please select only a secure, encrypted location'+
      ' to save this data.','Security Reminder',MB_OK + MB_DEFBUTTON1);


    if buttonSelected = ID_OK  then
      begin
        if SelectDirectory(DIR, [sdAllowCreate, sdPerformCreate, sdPrompt],1) then
        begin
          ExcelFile := DIR+'\'+edtBegDate.Text+'_'+edtEndDate.Text+'.csv';

          for i := 1 to grdMain.RowCount do  // revert blank dates back to blank
          begin
            if grdMain.Cells[1,i] = '01/01/0001' then grdMain.Cells[1,i] := '';
            if grdMain.Cells[2,i] = '01/01/0001' then grdMain.Cells[2,i] := '';
          end;
          grdMain.SaveToCSV(ExcelFile);
          ShellExecute(frmViewBilling.Handle, nil, PChar(ExcelFile), nil, nil, SW_SHOWNORMAL);
          lblExcelPath.Caption := DIR;

          for i := 1 to grdMain.RowCount do  // make blank dates sortable
          begin
            if grdMain.Cells[1,i] = '' then grdMain.Cells[1,i] := '01/01/0001';
            if grdMain.Cells[2,i] = '' then grdMain.Cells[2,i] := '01/01/0001';
          end;
        end;
      end;
  end;
  end;
  end;
end;

procedure TfrmViewBilling.btnCloseClick(Sender: TObject);
begin
Close;
end;

procedure TfrmViewBilling.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     if fileexists(excelfile) then         //modified with RMPR*149 to add securtiy enhancements
    begin
      if SysUtils.DeleteFile(ExcelFile) = False then
        raise Exception.Create('Please close the temporary Excel (.csv) file before navigating from this form')    //RMPR*136
      else
        lblExcelPath.Caption := '';
    end;
end;


procedure TfrmViewBilling.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
//if (ssAlt in Shift) and (chr(Key) in ['D','d']) then btnDisplay.Click;
//if (ssAlt in Shift) and (chr(Key) in ['P','p']) then btnPrintGrid.Click;
//if (ssAlt in Shift) and (chr(Key) in ['X','x']) then btnExcel.Click;
if (ssCtrl in Shift) and (chr(Key) in ['C','c']) then btnClose.Click;
end;

procedure TfrmViewBilling.mnuItemBegDateClick(Sender: TObject);
begin
dtpkrBegDate.Perform(WM_KEYDOWN, VK_F4, 0); //Calls Calendar from Menu
dtpkrBegDate.Perform(WM_KEYUP, VK_F4, 0);
end;

procedure TfrmViewBilling.mnuItemDisplayClick(Sender: TObject);
begin
btnDisplay.Click;
end;

procedure TfrmViewBilling.menuItemPrintClick(Sender: TObject);
begin
btnPrintGrid.Click;
end;

procedure TfrmViewBilling.mnuItemExcelClick(Sender: TObject);
begin
btnExcel.Click;
end;

procedure TfrmViewBilling.FormCreate(Sender: TObject);

begin

   CheckSec508;
   if Sec508 = '1' then
   begin
     imgGradient.EndColor   := clWhite;   //Change colors
     imgGradient.StartColor := clWhite;
     bnrMain.Font.Color     := clBlack;
     lblBegDate.Font.Color  := clBlack;
     lblEndDate.Font.Color  := clBlack;
//     bnrExclude.Font.Color  := clBlack;
     lblRecFound.Font.Color := clBlack;
     btnDisplay.Font.Color  := clBlack;
     btnExcel.Font.Color    := clBlack;

              grdMain.FixedColor := clWhite;
                   grdMain.Color := clWhite;
     grdMiniInsurance.FixedColor := clWhite;
          grdMiniInsurance.Color := clWhite;
           grdMiniDis.FixedColor := clWhite;
                grdMiniDis.Color := clWhite;
//     grdMain.Options := grdMain.Options-[goRowSelect]; //Turn Off
  end;
end;

procedure TfrmViewBilling.mnuItemAboutClick(Sender: TObject);
begin
dlgAbout.ShowModal;
end;

procedure TfrmViewBilling.mnuItemContentsClick(Sender: TObject);
begin
Application.HelpCommand (Help_Index,0);
end;

procedure TfrmViewBilling.mnuItemExitClick(Sender: TObject);
begin
Close;
end;

procedure TfrmViewBilling.mnuItemEndDateClick(Sender: TObject);
begin
dtpkrEndDate.Perform(WM_KEYDOWN, VK_F4, 0); //Calls Calendar from menu
dtpkrEndDate.Perform(WM_KEYUP, VK_F4, 0);
end;

procedure TfrmViewBilling.dtpkrBegDateKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if edtBegDate.text = '' then  //Checks if date had been selected, else
 dtpkrBegDate.Date := Now;  //Sets date from Menu call
end;

procedure TfrmViewBilling.dtpkrEndDateKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);     //Sets date from Menu call
begin                      //Needed when calling from Menu ?!?
 if edtEndDate.text = '' then
 dtpkrEndDate.Date := dtpkrBegDate.Date;
end;

procedure TfrmViewBilling.dtpkrBegDateEnter(Sender: TObject);
begin

  if edtBegDate.text = '' then   //Checks if date had been selected, else
  dtpkrBegDate.Date := Now;     //force today date
end;

procedure TfrmViewBilling.dtpkrEndDateEnter(Sender: TObject);
begin
  if edtEndDate.text = '' then   //Checks if date had been selected, else
  dtpkrEndDate.Date := dtpkrBegDate.Date;      //force today date
end;

procedure TfrmViewBilling.FormDestroy(Sender: TObject);
begin
Application.HelpCommand(HELP_QUIT, 0); //Close Help if still open

end;

procedure TfrmViewBilling.mnuItemSec508Click(Sender: TObject);
var
  reg : TRegistry;          //Toggle Sec 508 screen settings in registry
begin
  reg := TRegistry.Create;
  reg.rootkey := HKEY_LOCAL_MACHINE;
  reg.OpenKey('\SOFTWARE\VISTA\PROS\ViewBilling',False);
  Sec508 := reg.ReadString('Sec508');
   begin  //Toggle the registry key
    if Sec508 = '0' then
       begin
        if MessageDlg('Do you wish to activate Section 508 assistance?',
        mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          begin
            reg.WriteString('Sec508','1');    //Turn it on
            ShowMessage('Please EXIT and RESTART the program for the changes to take effect');
          end;
       end
   else
      begin
       if MessageDlg('Do you wish to turn off Section 508 assistance?',
       mtConfirmation, [mbYes, mbNo], 0) = mrYes then
         begin
           reg.WriteString('Sec508', '0');  //Turn it off
           ShowMessage('Please EXIT and RESTART the program for the changes to take effect');
         end;
      end;
 end;
  reg.CloseKey;
  reg.Free;


end;

procedure TfrmViewBilling.grdMainGetCellFormat(Sender: TObject; Col, Row: Integer;
  State: TGridDrawState; var FormatOptions: TFormatOptions; var CheckBox,
  Combobox, Ellipsis: Boolean);
begin
if (Col=8)  and (Row >0) then   //Sets QTY column for proper sorting
   formatoptions.AlignmentHorz := taRightJustify;
if (Col=9) and (Row >0) then   //Sets COST column for proper sorting
   formatoptions.AlignmentHorz := taRightJustify;
end;

procedure TfrmViewBilling.grdMainGetSortStyle(Sender: TObject; Col: Integer;
  var SortStyle: TSortStyle);
begin
 if Col= 1  then SortStyle := ssDateTime; //Sort DATE as a date
 if Col= 2  then SortStyle := ssDateTime; //Sort DATE as a date
 if Col= 8  then SortStyle := ssNumeric;  //Sort Qty as a mumber
 if Col= 9  then SortStyle := ssNumeric ; //Sort COST as a number
end;

procedure TfrmViewBilling.btnClearClick(Sender: TObject);
var
  i : integer;
begin
  lblDisRecFound.Caption := '0';
  for i :=1 to grdMain.RowCount -1 do   //If selected will clear the grid
   grdMain.Rows[i].Text := '';
   grdMain.Row := 1;
   grdMain.Cells[0,1] :='';
   grdMain.RowCount := 16;
end;

procedure TfrmViewBilling.grdMainDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
    i : integer;
begin
  with grdMain do
  begin
  // Draw the Bands with alternating colors
    if Sec508 = '0' then  //Sec508 check
     begin
     if ARow mod 2 = 0 then
        Canvas.Brush.Color := $CFFFFF    //light yellow
       else
        Canvas.Brush.Color := $FFFFFF;   //white
      end; //Sec508 check

  // Don't change color for first Column, first row
    if {(ACol = 0) or} (ARow = 0) then
      Canvas.Brush.Color := clBtnFace
    else
    begin
      case ACol of

        1: for i := 0 to grdMain.RowCount do //Sets cell color
            begin
            if (ARow = i) and (Cells[1,i] = '01/01/0001') then
             begin
               if ARow mod 2 = 0 then
               Canvas.Font.Color := $CFFFFF    //light yellow
              else
               Canvas.Font.Color := $FFFFFF;   //white

             end;
            end;

        2: for i := 0 to grdMain.RowCount do //Sets cell color
            begin
            if (ARow = i) and (Cells[2,i] = '01/01/0001') then
             begin
               if ARow mod 2 = 0 then
               Canvas.Font.Color := $CFFFFF    //light yellow
              else
               Canvas.Font.Color := $FFFFFF;   //white
             end;
    //          if gdSelected in State then Canvas.Font.Color := clActiveCaption;
           end;


        10: for i := 1 to grdMain.RowCount do //Sets cell color
            begin
             if (ARow = i) and ((Cells[15,i] = '13') or (Cells[15,i] = '132')) then
              begin
               Canvas.Font.Color := clRed;
               Canvas.Font.Style := [fsBold];
//               Canvas.Brush.Color := clRed ;
              end;
            end;

      11: for i := 1 to grdMain.RowCount do //Sets cell color
            begin
             if (ARow = i) and ((Cells[15,i] = '13') or (Cells[15,i] = '132')) then
              begin
               Canvas.Font.Color := clRed;
               Canvas.Font.Style := [fsBold];
//               Canvas.Brush.Color := clRed ;
              end;
            end;

        12: for i := 1 to grdMain.RowCount do //Sets cell color
            begin
             if (ARow = i) and ((Cells[15,i] = '12') or (Cells[15,i] = '132') or (Cells[15,i] = '142')) then
              begin
               Canvas.Font.Color := clRed;
               Canvas.Font.Style := [fsBold];
//               Canvas.Brush.Color := clRed ;
              end;
            end;

      13: for i := 1 to grdMain.RowCount do //Sets cell color
            begin
             if (ARow = i) and ((Cells[15,i] = '12') or (Cells[15,i] = '132') or (Cells[15,i] = '142')) then
              begin
               Canvas.Font.Color := clRed;
               Canvas.Font.Style := [fsBold];
//               Canvas.Brush.Color := clRed ;
              end;
            end;


{
      9: for i := 0 to grdMain.RowCount do //Sets cell color
            begin
             if Cells[6,i] <> '' then
             begin
            if (ARow = i) and (StrToInt(Cells[6,i]) > 0) then  //First level
             begin
               Canvas.Font.Color := clBlack;
               Canvas.Font.Style := [fsBold];
               //Canvas.Brush.Color := clYellow;
             end;
          end;
         end;

        7: for i := 0 to grdMain.RowCount do //Sets cell color
            begin
             if Cells[7,i] <> '' then
             begin
            if (ARow = i) and (StrToInt(Cells[7,i]) > 0) then
             begin
               Canvas.Font.Color := clYellow;
               Canvas.Font.Style := [fsBold];
               Canvas.Brush.Color := clRed;
             end;
            end;
          end;


}     end;


// Set color for selected row
  if gdSelected in State then
  begin
    Canvas.Brush.Color := clActiveCaption;
    Canvas.Font.Color  := clCaptionText;
        Case ACol of
              2: if (Cells[2,ARow] = '01/01/0001') then
                 Canvas.Font.Color := clActiveCaption;
        end;
   end;

//Final draw of grid
      Canvas.TextRect(Rect, Rect.Left + 2, Rect.Top + 2, cells[acol, arow]);
      Canvas.FrameRect(Rect);

    end;
  end;
end;

procedure TfrmViewBilling.btnPtDisClick(Sender: TObject);
var
  i : integer;
begin
    frmMenu.brkrRPC.RemoteProcedure := 'RMPR DIS LIST 660';   //Pt Disability Grid
    frmMenu.brkrRPC.Param[0].Value := IEN660;
    frmMenu.brkrRPC.Param[0].PType := literal;
  TRY
    frmMenu.brkrRPC.Call;
  Except
   on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
  END;

 if frmMenu.brkrRPC.Results.Count > 0 then
 begin
  grdMiniDis.RowCount := frmMenu.brkrRPC.Results.Count +1;//Sizes grid to match results

 for i:= 0 to frmMenu.brkrRPC.Results.Count -1 do
   begin
    with grdMiniDis do
      begin
         Cells[0,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',1);//Disab
         Cells[1,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',2);//% Disab
         Cells[2,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',3);// SC YES/NO

      end; // with do

    end; //Result check
   end
  else grdMiniDis.RowCount := 1;
end;


procedure TfrmViewBilling.grdMainClick(Sender: TObject);
begin
IEN660 := '';
IEN660 := grdMain.Cells[14,grdMain.Row];
if IEN660 <> '' then
 begin
   btnPtDis.Click;
   btnMiniInsurance.Click;
 end;
end;

procedure TfrmViewBilling.btnMiniInsuranceClick(Sender: TObject);
var
  i : integer;
begin
    frmMenu.brkrRPC.RemoteProcedure := 'RMPR PFFS INS';   //Pt Disability Grid
    frmMenu.brkrRPC.Param[0].Value := IEN660;
    frmMenu.brkrRPC.Param[0].PType := literal;
  TRY
    frmMenu.brkrRPC.Call;
  Except
   on E:EBrokerError do
       begin
       Showmessage('Broker Error: '+ E.Message);
       end;
     on E:EAccessViolation do
       Showmessage('E.Message');
     else
       raise;
  END;

 if frmMenu.brkrRPC.Results.Count > 0 then
 begin
 grdMiniInsurance.RowCount := frmMenu.brkrRPC.Results.Count +1;//Sizes grid to match results

 for i:= 0 to frmMenu.brkrRPC.Results.Count -1 do
    begin
    with grdMiniInsurance do
      begin
         Cells[0,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',1);//Insurance
         Cells[1,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',7);//COB
         Cells[2,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',2);//Sub ID
         Cells[3,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',3);//Group
         Cells[4,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',4);//Holder
         Cells[5,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',6);//Effective Date
           if Cells[5,i+1] = '' then Cells[5,i+1] := '01/01/0001'; //Adds date if none
         Cells[6,i+1] := Piece(frmMenu.brkrRPC.Results[i],'^',5);//Expire Date
           if Cells[6,i+1] = '' then Cells[6,i+1] := '01/01/0001'; //Adds date if none

      end; // with do
    end; //Results Count check
  end
       else grdMiniInsurance.RowCount := 1;
end;


procedure TfrmViewBilling.grdMiniInsuranceGetSortStyle(Sender: TObject;
  Col: Integer; var SortStyle: TSortStyle);
begin
 if Col= 5  then SortStyle := ssDateTime; //Sort DATE as a date
 if Col= 6  then SortStyle := ssDateTime; //Sort DATE as a date
end;

procedure TfrmViewBilling.grdMiniInsuranceDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
    i : integer;
begin
  with grdMiniInsurance do
  begin

  // Don't change color for first Column, first row
    if {(ACol = 0) or} (ARow = 0) then
      Canvas.Brush.Color := clBtnFace
    else
    begin
      case ACol of

        5: for i := 0 to grdMiniInsurance.RowCount do //Sets cell color
            begin
              if (ARow = i) and (Cells[5,i] = '01/01/0001') then
               Canvas.Font.Color := $CFFFFF;    //light yellow
            end;
         
        6: for i := 0 to grdMiniInsurance.RowCount do //Sets cell color
            begin
              if (ARow = i) and (Cells[6,i] = '01/01/0001') then
               Canvas.Font.Color := $CFFFFF;    //light yellow
            end;
         end;
   // end;


// Set color for selected row
  if gdSelected in State then
  begin
    Canvas.Brush.Color := clActiveCaption;
    Canvas.Font.Color  := clCaptionText;
      Case Acol of
        5: if (Cells[5,ARow] = '01/01/0001') then
              Canvas.Font.Color := clActiveCaption;
        6: if (Cells[6,ARow] = '01/01/0001') then
              Canvas.Font.Color := clActiveCaption;
   end;
  end;

//Final draw of grid
      Canvas.TextRect(Rect, Rect.Left + 2, Rect.Top + 2, cells[acol, arow]);
      Canvas.FrameRect(Rect);
   end;
  end;
end;

end.
