

unit progbar;

interface

uses
  Classes, Windows, Controls, Graphics;

type
  TAnimationThread = class(TThread)
  private
    { Private declarations }
    FWnd: HWND;
    FPaintRect: TRect;
    FbkColor, FfgColor: TColor;
    FInterval: Integer;
  protected
    procedure Execute; override;
  public
    constructor create(
      paintsurface: TWinControl;  {Control to paint on}
      paintrect: TRect;  {area for animation bar}
      bkColor, barcolor: TColor;  {colors to use}
      interval: Integer  {wait in msecs between paints}
      );
end;

implementation

constructor TAnimationThread.create(paintsurface: TWinControl;
paintrect: TRect; bkColor, barcolor: TColor; interval: Integer);
begin
  inherited Create( true );
  FWnd := paintsurface.Handle;
  FPaintRect := paintrect;
  FbkColor := bkColor;
  FfgColor := barColor;
  FInterval := interval;
  FreeOnterminate := True;
  Resume;
end;


procedure TAnimationThread.Execute;
var
  image: TBitmap;
  DC: HDC;
  left, right: Integer;
  increment: Integer;
  imagerect: TRect;
  state : (incRight, incLeft, decLeft, decRight);
begin
  Image := TBitmap.Create;
  try
    with Image do
    begin
      Width  := FPaintRect.Right - FPaintRect.Left;
      Height := FPaintRect.Bottom - FPaintRect.Top;
      imagerect := Rect(0, 0, Width, Height);
    end;
    left := 0;
    right := 0;
    increment := imagerect.right div 50;
    state := Low(State);
    while not Terminated do
    begin
      with Image.Canvas do
      begin
        Brush.Color := FbkColor;
        FillRect( imagerect );
        Case state of
          incRight:
          begin
            Inc( right, increment );
            if right > imagerect.right then
            begin
              right := imagerect.right;
              Inc(state);
            end;
          end;
          incLeft:
          begin
            Inc( left, increment );
            if left >= right then
            begin
              left := right;
              Inc(state);
            end;
          end;
          decLeft:
          begin
            Dec( left, increment );
            if left <= 0 then
            begin
              left := 0;
              Inc(state);
            end;
          end;
          decRight:
          begin
            Dec( right, increment );
            if right <= 0 then
            begin
              right := 0;
              state := incRight;
            end;
          end;
        end;
        Brush.Color := FfgColor;
        FillRect( Rect( left, imagerect.top, right, imagerect.bottom ));
      end;
      DC := GetDC( FWnd );
      if DC <> 0 then
        try
          BitBlt( DC, FPaintRect.Left, FPaintRect.Top, imagerect.right, imagerect.bottom,
                      Image.Canvas.handle, 0, 0, SRCCOPY );
        finally
          ReleaseDC( FWnd, DC );
        end;
      Sleep( FInterval );
    end;
  finally
    Image.Free;
  end;
  InvalidateRect( FWnd, Nil, true );
end;

end.


