unit oVersionInformation;
{
================================================================================
*
*       Application:  Clinical Observations
*       Revision:     $Revision: 1 $  $Modtime: 8/06/09 8:03a $
*       Developer:    auser@domain.ext
*       Site:         Hines OIFO
*
*       Description:  Generic Version Information Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /PROS/Prosthetics Vista Suite/Source/Components/Version/oVersion/oVersionInformation.pas $
*
* $History: oVersionInformation.pas $
 * 
 * *****************  Version 1  *****************
 * User: Vhaishnelsok Date: 1/07/11    Time: 11:57a
 * Created in $/PROS/Prosthetics Vista Suite/Source/Components/Version/oVersion
 * 
 * *****************  Version 1  *****************
 * User: Vhaishnelsok Date: 8/05/09    Time: 8:09p
 * Created in $/PROS/Prosthetics Vista Suite V4/Source/Components/Version/oVersion
 * 
 * *****************  Version 1  *****************
 * User: Vhaishpetitd Date: 10/05/06   Time: 12:13p
 * Created in $/CP Modernization/CliO/source/Common/objects
 * Initial Check-In
 *
*
================================================================================
}
interface

uses
  SysUtils,
  WinTypes,
  //Dialogs,
  Classes;

type
  { exception classes for version info }
  EVerInfoError = class(Exception)
  end;

  ENoVerInfoError = class(Exception)
  end;

  ENoFixeVerInfo = class(Exception)
  end;

  { different types of pre-defined version info }
  TVerInfoType = (
    viCompanyName,
    viFileDescription,
    viFileVersion,
    viInternalName,
    viLegalCopyright,
    viLegalTrademarks,
    viOriginalFilename,
    viProductName,
    viProductVersion,
    viComments);

const
  {pre-defined version information keys}
  VerNameArray: array[viCompanyName..viComments] of string[20] = (
    'CompanyName',
    'FileDescription',
    'FileVersion',
    'InternalName',
    'LegalCopyright',
    'LegalTrademarks',
    'OriginalFilename',
    'ProductName',
    'ProductVersion',
    'Comments');

type
  TCrc32Table = array[0..255] of Int64;

  TBuffer = array[0..1048576] of byte;

  TVersionInformation = class(TObject)
  private
    fFileFlags: TStringList;
    fFileName: string;
    FixedFileInfoBuf: PVSFixedFileInfo;
    fNameSpace: string;
    fVersionMajor: Integer;
    fVersionMinor: Integer;
    fVersionPatch: Integer;
    fVersionTest: Integer;
    Handle: DWord;
    RezBuffer: string;
    Size: Integer;
    TransTable: PLongint;
  protected
    procedure FillFileMaskInfo;
    procedure FillFileVersionInfo;
    procedure FillFixedFileInfoBuf;
  public
    constructor Create(AFileName: string; ANameSpace: string = '');
    destructor Destroy; override;
    function getComments: string;
    function getCompanyName: string;
    function getCompiledDateTime: string;
    function getCRC32: string;
    function getDebugBuild: Boolean;
    function getFileDescription: string;
    function getFileFlags: string;
    function getFileNameAndVersion: string;
    function getFileOS: string;
    function getFileVersion: string;
    function getInternalName: string;
    function getLegalCopyright: string;
    function getLegalTrademarks: string;
    function getOriginalFileName: string;
    function getPreDefKeyString(AVerKind: TVerInfoType): string;
    function getPreRelease: Boolean;
    function getPrivateBuild: Boolean;
    function getProductName: string;
    function getProductVersion: string;
    function getSpecialBuild: Boolean;
    function getUserDefinedKey(AKey: string): string;
    function getVAPatchNumber: string;
    function getVAReleaseDate: string;
    function getVAVersionNumber: string;

    property VersionMajor: integer read fVersionMajor;
    property VersionMinor: integer read fVersionMinor;
    property VersionRelease: integer read fVersionPatch;
    property VersionBuild: integer read fVersionTest;
  end;

implementation

uses Windows;

const
  // strings that must be fed to VerQueryValue() function
  SFInfo = '\StringFileInfo\';
  VerTranslation: PChar = '\VarFileInfo\Translation';
  FormatStr = '%s%.4x%.4x\%s%s';

  BUFFERSIZE: integer = 65536;

  CRC32TABLE: TCrc32Table =
  ($0, $77073096, $EE0E612C, $990951BA,
    $76DC419, $706AF48F, $E963A535, $9E6495A3,
    $EDB8832, $79DCB8A4, $E0D5E91E, $97D2D988,
    $9B64C2B, $7EB17CBD, $E7B82D07, $90BF1D91,
    $1DB71064, $6AB020F2, $F3B97148, $84BE41DE,
    $1ADAD47D, $6DDDE4EB, $F4D4B551, $83D385C7,
    $136C9856, $646BA8C0, $FD62F97A, $8A65C9EC,
    $14015C4F, $63066CD9, $FA0F3D63, $8D080DF5,
    $3B6E20C8, $4C69105E, $D56041E4, $A2677172,
    $3C03E4D1, $4B04D447, $D20D85FD, $A50AB56B,
    $35B5A8FA, $42B2986C, $DBBBC9D6, $ACBCF940,
    $32D86CE3, $45DF5C75, $DCD60DCF, $ABD13D59,
    $26D930AC, $51DE003A, $C8D75180, $BFD06116,
    $21B4F4B5, $56B3C423, $CFBA9599, $B8BDA50F,
    $2802B89E, $5F058808, $C60CD9B2, $B10BE924,
    $2F6F7C87, $58684C11, $C1611DAB, $B6662D3D,
    $76DC4190, $1DB7106, $98D220BC, $EFD5102A,
    $71B18589, $6B6B51F, $9FBFE4A5, $E8B8D433,
    $7807C9A2, $F00F934, $9609A88E, $E10E9818,
    $7F6A0DBB, $86D3D2D, $91646C97, $E6635C01,
    $6B6B51F4, $1C6C6162, $856530D8, $F262004E,
    $6C0695ED, $1B01A57B, $8208F4C1, $F50FC457,
    $65B0D9C6, $12B7E950, $8BBEB8EA, $FCB9887C,
    $62DD1DDF, $15DA2D49, $8CD37CF3, $FBD44C65,
    $4DB26158, $3AB551CE, $A3BC0074, $D4BB30E2,
    $4ADFA541, $3DD895D7, $A4D1C46D, $D3D6F4FB,
    $4369E96A, $346ED9FC, $AD678846, $DA60B8D0,
    $44042D73, $33031DE5, $AA0A4C5F, $DD0D7CC9,
    $5005713C, $270241AA, $BE0B1010, $C90C2086,
    $5768B525, $206F85B3, $B966D409, $CE61E49F,
    $5EDEF90E, $29D9C998, $B0D09822, $C7D7A8B4,
    $59B33D17, $2EB40D81, $B7BD5C3B, $C0BA6CAD,
    $EDB88320, $9ABFB3B6, $3B6E20C, $74B1D29A,
    $EAD54739, $9DD277AF, $4DB2615, $73DC1683,
    $E3630B12, $94643B84, $D6D6A3E, $7A6A5AA8,
    $E40ECF0B, $9309FF9D, $A00AE27, $7D079EB1,
    $F00F9344, $8708A3D2, $1E01F268, $6906C2FE,
    $F762575D, $806567CB, $196C3671, $6E6B06E7,
    $FED41B76, $89D32BE0, $10DA7A5A, $67DD4ACC,
    $F9B9DF6F, $8EBEEFF9, $17B7BE43, $60B08ED5,
    $D6D6A3E8, $A1D1937E, $38D8C2C4, $4FDFF252,
    $D1BB67F1, $A6BC5767, $3FB506DD, $48B2364B,
    $D80D2BDA, $AF0A1B4C, $36034AF6, $41047A60,
    $DF60EFC3, $A867DF55, $316E8EEF, $4669BE79,
    $CB61B38C, $BC66831A, $256FD2A0, $5268E236,
    $CC0C7795, $BB0B4703, $220216B9, $5505262F,
    $C5BA3BBE, $B2BD0B28, $2BB45A92, $5CB36A04,
    $C2D7FFA7, $B5D0CF31, $2CD99E8B, $5BDEAE1D,
    $9B64C2B0, $EC63F226, $756AA39C, $26D930A,
    $9C0906A9, $EB0E363F, $72076785, $5005713,
    $95BF4A82, $E2B87A14, $7BB12BAE, $CB61B38,
    $92D28E9B, $E5D5BE0D, $7CDCEFB7, $BDBDF21,
    $86D3D2D4, $F1D4E242, $68DDB3F8, $1FDA836E,
    $81BE16CD, $F6B9265B, $6FB077E1, $18B74777,
    $88085AE6, $FF0F6A70, $66063BCA, $11010B5C,
    $8F659EFF, $F862AE69, $616BFFD3, $166CCF45,
    $A00AE278, $D70DD2EE, $4E048354, $3903B3C2,
    $A7672661, $D06016F7, $4969474D, $3E6E77DB,
    $AED16A4A, $D9D65ADC, $40DF0B66, $37D83BF0,
    $A9BCAE53, $DEBB9EC5, $47B2CF7F, $30B5FFE9,
    $BDBDF21C, $CABAC28A, $53B39330, $24B4A3A6,
    $BAD03605, $CDD70693, $54DE5729, $23D967BF,
    $B3667A2E, $C4614AB8, $5D681B02, $2A6F2B94,
    $B40BBE37, $C30C8EA1, $5A05DF1B, $2D02EF8D);

{
************************************************* TVersionInformation **************************************************
}

constructor TVersionInformation.Create(AFileName: string; ANameSpace: string = '');
begin
  fFileName := aFileName;
  fNameSpace := aNameSpace;
  fFileFlags := TStringList.Create;
  FillFileVersionInfo;
  FillFixedFileInfoBuf;
  FillFileMaskInfo;
  with FixedFileInfoBuf^ do
    begin
      fVersionMajor := HIWORD(dwFileVersionMS);
      fVersionMinor := LOWORD(dwFileVersionMS);
      fVersionPatch := HIWORD(dwFileVersionLS);
      fVersionTest := LOWORD(dwFileVersionLS);
    end;
end;

destructor TVersionInformation.Destroy;
begin
  fFileFlags.Free;
end;

procedure TVersionInformation.FillFileMaskInfo;
begin
  with FixedFileInfoBuf^ do
    begin
      if (dwFileFlagsMask and dwFileFlags and VS_FF_PRERELEASE) <> 0 then
        fFileFlags.Add('Pre-release');
      if (dwFileFlagsMask and dwFileFlags and VS_FF_PRIVATEBUILD) <> 0 then
        fFileFlags.Add('Private build');
      if (dwFileFlagsMask and dwFileFlags and VS_FF_SPECIALBUILD) <> 0 then
        fFileFlags.Add('Special build');
      if (dwFileFlagsMask and dwFileFlags and VS_FF_DEBUG) <> 0 then
        fFileFlags.Add('Debug');
    end;
end;

procedure TVersionInformation.FillFileVersionInfo;
var
  SBSize: UInt;
begin
  // Determine size of version information
  Size := GetFileVersionInfoSize(PChar(FFileName), Handle);
  if Size <= 0 then { raise exception if size <= 0 }
    raise ENoVerInfoError.Create('No Version Info Available.');

  // Set the length accordingly
  SetLength(RezBuffer, Size);
  // Fill the buffer with version information, raise exception on error
  if not GetFileVersionInfo(PChar(FFileName), Handle, Size, PChar(RezBuffer)) then
    raise EVerInfoError.Create('Cannot obtain version info.');

  // Get translation info, raise exception on error
  if not VerQueryValue(PChar(RezBuffer), VerTranslation, pointer(TransTable),
    SBSize) then
    raise EVerInfoError.Create('No language info.');
end;

procedure TVersionInformation.FillFixedFileInfoBuf;
var
  Size: Cardinal;
begin
  if VerQueryValue(PChar(RezBuffer), '\', Pointer(FixedFileInfoBuf), Size) then
    begin
      if Size < SizeOf(TVSFixedFileInfo) then
        raise eNoFixeVerInfo.Create('No fixed file info');
    end
  else
    raise eNoFixeVerInfo.Create('No fixed file info')
end;

function TVersionInformation.getComments: string;
begin
  Result := getPreDefKeyString(viComments);
end;

function TVersionInformation.getCompanyName: string;
begin
  Result := getPreDefKeyString(viCompanyName);
end;

function TVersionInformation.getCompiledDateTime: string;
begin
  Result := FormatDateTime('mmmm d, yyyy hh:mm', FileDateToDateTime(FileAge(fFileName)));
end;

function TVersionInformation.getCRC32: string;
var
  Buffer: Pointer;
  Handle, Loaded: Integer;
  TempResult: DWord;

  function UpdateCrc32(Value: integer; var Buffer: array of byte; Count: integer): integer;
  var
    i: integer;
  begin
    Result := Value;
    for i := 0 to Count - 1 do
      begin
        Result := ((Result shr 8) and $00FFFFFF) xor CRC32TABLE[(Result xor Buffer[i]) and $000000FF];
      end;
  end;

begin
  TempResult := $FFFFFFFF;
  GetMem(Buffer, BUFFERSIZE);
  Handle := FileOpen(fFileName, fmShareDenyWrite);
  repeat
    Loaded := FileRead(Handle, Buffer^, BUFFERSIZE);
    TempResult := UpdateCrc32(TempResult, TBuffer(Buffer^), Loaded);
  until Loaded <> BUFFERSIZE;
  FileClose(Handle);
  FreeMem(Buffer);
  TempResult := not TempResult;
  Result := IntToHex(TempResult, 8);
end;

function TVersionInformation.getDebugBuild: Boolean;
begin
  Result := (fFileFlags.IndexOf('Debug') > -1);
end;

function TVersionInformation.getFileDescription: string;
begin
  Result := getPreDefKeyString(viFileDescription);
end;

function TVersionInformation.getFileFlags: string;
begin
  Result := fFileFlags.CommaText;
end;

function TVersionInformation.getFileNameAndVersion: string;
begin
  Result := UpperCase(ExtractFileName(fFileName)) + ':' + getFileVersion;
end;

function TVersionInformation.getFileOS: string;
begin
  with FixedFileInfoBuf^ do
    case dwFileOS of
      VOS_UNKNOWN: Result := 'Unknown';
      VOS_DOS: Result := 'Designed for MS-DOS';
      VOS_OS216: Result := 'Designed for 16-bit OS/2';
      VOS_OS232: Result := 'Designed for 32-bit OS/2';
      VOS_NT: Result := 'Designed for Windows NT';

      VOS__WINDOWS16: Result := 'Designed for 16-bit Windows';
      VOS__PM16: Result := 'Designed for 16-bit PM';
      VOS__PM32: Result := 'Designed for 32-bit PM';
      VOS__WINDOWS32: Result := 'Designed for 32-bit Windows';

      VOS_DOS_WINDOWS16: Result := 'Designed for 16-bit Windows, running on MS-DOS';
      VOS_DOS_WINDOWS32: Result := 'Designed for Win32 API, running on MS-DOS';
      VOS_OS216_PM16: Result := 'Designed for 16-bit PM, running on 16-bit OS/2';
      VOS_OS232_PM32: Result := 'Designed for 32-bit PM, running on 32-bit OS/2';
      VOS_NT_WINDOWS32: Result := 'Designed for Win32 API, running on Windows/NT';
    else
      Result := 'Unknown';
    end;
end;

function TVersionInformation.getFileVersion: string;
begin
  Result := Format('%d.%d.%d.%d', [fVersionMajor, fVersionMinor, fVersionPatch, fVersionTest]);
end;

function TVersionInformation.getInternalName: string;
begin
  Result := getPreDefKeyString(viInternalName);
end;

function TVersionInformation.getLegalCopyright: string;
begin
  Result := getPreDefKeyString(viLegalCopyright);
end;

function TVersionInformation.getLegalTrademarks: string;
begin
  Result := getPreDefKeyString(viLegalTrademarks);
end;

function TVersionInformation.getOriginalFileName: string;
begin
  Result := getPreDefKeyString(viOriginalFilename);
end;

function TVersionInformation.getPreDefKeyString(AVerKind: TVerInfoType): string;
var
  P: PChar;
  S: UInt;
begin
  Result := Format(FormatStr, [SfInfo, LoWord(TransTable^), HiWord(TransTable^),
    VerNameArray[aVerKind], #0]);
  // get and return version query info, return empty string on error
  if VerQueryValue(PChar(RezBuffer), @Result[1], Pointer(P), S) then
    Result := StrPas(P)
  else
    Result := '';
end;

function TVersionInformation.getPreRelease: Boolean;
begin
  Result := (fFileFlags.IndexOf('Pre-release') > -1);
end;

function TVersionInformation.getPrivateBuild: Boolean;
begin
  Result := (fFileFlags.IndexOf('Private build') > -1);
end;

function TVersionInformation.getProductName: string;
begin
  Result := getPreDefKeyString(viProductName);
end;

function TVersionInformation.getProductVersion: string;
begin
  Result := Format('%d.%d.%d.%d', [fVersionMajor, fVersionMinor, fVersionPatch, fVersionTest]);
end;

function TVersionInformation.getSpecialBuild: Boolean;
begin
  Result := (fFileFlags.IndexOf('Special build') > -1);
end;

function TVersionInformation.getUserDefinedKey(AKey: string): string;
var
  P: PChar;
  S: UInt;
begin
  Result := Format(FormatStr, [SfInfo, LoWord(TransTable^), HiWord(TransTable^), aKey, #0]);
  if VerQueryValue(PChar(RezBuffer), @Result[1], Pointer(P), S) then
    Result := StrPas(P)
  else
    Result := '';
end;

function TVersionInformation.getVAPatchNumber: string;
begin
  if fVersionPatch = 0 then
    Result := ''
  else
    Result := Format('%s*%d.%d*%d', [fNameSpace, fVersionMajor, fVersionMinor, fVersionPatch]);
  (*
  begin
    if fVersionTest > 0 then
    Result := Format('%sT%d', [Result, fVersionTest]);
  end;
  *)
end;

function TVersionInformation.getVAReleaseDate: string;
begin
  Result := getUserDefinedKey('VAReleaseDate');
end;

function TVersionInformation.getVAVersionNumber: string;
begin
  Result := Format('%s %d.%d', [fNameSpace, fVersionMajor, fVersionMinor]);
  (*
  if (fVersionTest > 0) and (fVersionPatch = 0) then
    Result := Format('%sT%d', [Result, fVersionTest]);
  *)
end;

{Protected Methods}

{Public getter methods}

end.

