unit uPROS_CDSCommon;

interface
    uses
    db
    , dbClient
    , classes;

const
  fwDateTime = 12;
  fwFloat = 12;
  fwBoolean = 5;
  gMDK_DateFMT = 'mm/dd/yyyy';
  gMDK_TimeFMT = 'hh:nn:ss';
  gMDK_DateTimeFMT = 'mm/dd/yyyy hh:nn:ss';
  gMDK_DebugTimeFMT = 'hh:MM:ss.zzz';
  al = taLeftJustify;
  ar = taRightJustify;
  ac = taCenter;



Type
TCreateCDS = Procedure;


function NewAggregateField(aCDS: TClientDataSet; aFieldname, anExpression:
  string): TAggregateField;
function NewIntegerField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer): TIntegerField;
function NewFloatField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aDisplayFormat: string = '#.##'; aWidth: Integer = fwFloat): TFloatField;
function NewBooleanField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwBoolean): TBooleanField;
function NewStringField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer; aSize: Integer): TStringField;
function NewDateTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateFMT): TDateTimeField;
function NewTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateTimeFMT): TDateTimeField;




var
  LastErrorMessage: String;

implementation

function NewAggregateField(aCDS: TClientDataSet; aFieldname, anExpression:
  string): TAggregateField;
var
  aF: TAggregateField;
begin
  Result := nil;
  try
    aCDS.Close;
    aF := TAggregateField.Create(aCDS);
    aF.DisplayName := aFieldName;
    aF.Name := aCDS.Name + aFieldName;
    aF.Expression := anExpression;
    aF.Visible := True;
    aF.ReadOnly := True;
    aF.Active := True;
    aF.GroupingLevel := 1;
    Result := aF;
  except
  end;
end;

function NewIntegerField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer): TIntegerField;
var
  fF: TIntegerField;
begin
  aCDS.Close;
  fF := TIntegerField.Create(aCDS);
  fF.FieldName := aFieldName;
  fF.Name := aCDS.Name + fF.FieldName;
  fF.Index := aCDS.FieldCount;
  fF.DataSet := aCDS;
  fF.DisplayLabel := aLabel;
  fF.DisplayWidth := aWidth;

  fF.Alignment := al;

  Result := fF;
end;

function NewFloatField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aDisplayFormat: string = '#.##'; aWidth: Integer = fwFloat): TFloatField;
var
  fF: TFloatField;
begin
  aCDS.Close;
  fF := TFloatField.Create(aCDS);
  fF.FieldName := aFieldName;
  fF.Name := aCDS.Name + fF.FieldName;
  fF.Index := aCDS.FieldCount;
  fF.DataSet := aCDS;
  fF.DisplayLabel := aLabel;
  fF.DisplayWidth := aWidth;
{$IFDEF CONTROL_FORMAT}
  fF.DisplayFormat := aDisplayFormat;
{$ENDIF}
  fF.Alignment := ar;

  Result := fF;
end;

function NewBooleanField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwBoolean): TBooleanField;
var
  fF: TBooleanField;
begin
  aCDS.Close;
  fF := TBooleanField.Create(aCDS);
  fF.FieldName := aFieldName;
  fF.Name := aCDS.Name + fF.FieldName;
  fF.Index := aCDS.FieldCount;
  fF.DataSet := aCDS;
  fF.DisplayLabel := aLabel;
  fF.DisplayWidth := aWidth;

  fF.Alignment := al;

  Result := fF;
end;

function NewStringField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer; aSize: Integer): TStringField;
var
  fS: TStringField;
begin
  aCDS.Close;
  fS := TStringField.Create(aCDS);
  fS.FieldName := aFieldName;
  fS.Name := aCDS.Name + fS.FieldName;
  fS.Index := aCDS.FieldCount;
  fS.DataSet := aCDS;
  fs.Size := aSize;
  fs.DisplayLabel := aLabel;
  fs.DisplayWidth := aWidth;

  fs.Alignment := al;

  Result := fS;
end;

function NewDateTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateFMT): TDateTimeField;
var
  fDT: TDateTimeField;
begin
  aCDS.Close;
  fDT := TDateTimeField.Create(aCDS);
  fDT.FieldName := aFieldName;
  fDT.Name := aCDS.Name + fDT.FieldName;
  fDT.Index := aCDS.FieldCount;
  fDT.DataSet := aCDS;
  fDT.DisplayLabel := aLabel;
  fDT.DisplayWidth := aWidth;
  fDT.DisplayFormat := aDisplayFMT;

  fDT.Alignment := al;

  Result := fDT;
end;

function NewTimeField(aCDS: TClientDataSet; aFieldName, aLabel: string;
  aWidth: Integer = fwDateTime;
  aDisplayFMT: string = gMDK_DateTimeFMT): TDateTimeField;
var
  fDT: TTimeField;
begin
  aCDS.Close;
  fDT := TTimeField.Create(aCDS);
  fDT.FieldName := aFieldName;
  fDT.Name := aCDS.Name + fDT.FieldName;
  fDT.Index := aCDS.FieldCount;
  fDT.DataSet := aCDS;
  fDT.DisplayLabel := aLabel;
  fDT.DisplayWidth := aWidth;
  fDT.DisplayFormat := aDisplayFMT;

  fDT.Alignment := al;

  Result := fDT;
end;


end.
